@@ -33,6 +33,7 @@
 #include "nepomukmanager.h"
 #include "debuginterface.h"
 #include "storage/itemretrievalthread.h"
+#include "preprocessormanager.h"
 
 #include "libs/xdgbasedirs_p.h"
 #include "libs/protocol_p.h"
@@ -62,7 +63,10 @@ AkonadiServer::AkonadiServer( QObject* parent )
     , mAlreadyShutdown( false )
 {
     DbConfig::init();
-    startDatabaseProcess();
+    if ( DbConfig::useInternalServer() )
+      startDatabaseProcess();
+    else
+      createDatabase();
 
     s_instance = this;
 
@@ -107,6 +111,14 @@ AkonadiServer::AkonadiServer( QObject* parent )
     Tracer::self();
     new DebugInterface( this );
     ResourceManager::self();
+
+    // Initialize the preprocessor manager
+    PreprocessorManager::init();
+
+    // Forcibly disable it if configuration says so
+    if ( settings.value( QLatin1String( "General/DisablePreprocessing" ), false ).toBool() )
+      PreprocessorManager::instance()->setEnabled( false );
+
     if ( settings.value( QLatin1String( "Cache/EnableCleaner" ), true ).toBool() ) {
       mCacheCleaner = new CacheCleaner( this );
       mCacheCleaner->start( QThread::IdlePriority );
@@ -134,6 +146,13 @@ AkonadiServer::AkonadiServer( QObject* parent )
 
     connect( QDBusConnection::sessionBus().interface(), SIGNAL(serviceOwnerChanged(QString,QString,QString)),
              SLOT(serviceOwnerChanged(QString,QString,QString)) );
+
+    // Unhide all the items that are actually hidden.
+    // The hidden flag was probably left out after an (abrupt)
+    // server quit. We don't attempt to resume preprocessing
+    // for the items as we don't actually know at which stage the
+    // operation was interrupted...
+    db->unhideAllPimItems();
 }
 
 AkonadiServer::~AkonadiServer()
@@ -169,6 +188,9 @@ void AkonadiServer::quit()
       quitThread( mConnections[ i ] );
     mConnections.clear();
 
+    // Terminate the preprocessor manager before the database but after all connections are gone
+    PreprocessorManager::done();
+
     DataStore::self()->close();
     Q_ASSERT( QSqlDatabase::connectionNames().isEmpty() );
 
@@ -361,6 +383,35 @@ void AkonadiServer::startDatabaseProcess()
   QSqlDatabase::removeDatabase( initCon );
 }
 
+void AkonadiServer::createDatabase()
+{
+  const QLatin1String initCon( "initConnection" );
+  QSqlDatabase db = QSqlDatabase::addDatabase( DbConfig::driverName(), initCon );
+  DbConfig::configure( db );
+  db.setDatabaseName( QString() ); // might not exist yet, then connecting to the actual db will fail
+  if ( !db.isValid() )
+    akFatal() << "Invalid database object during initial database connection";
+
+  if ( db.open() ) {
+    {
+      QSqlQuery query( db );
+      if ( !query.exec( QString::fromLatin1( "USE %1" ).arg( DbConfig::databaseName() ) ) ) {
+        akDebug() << "Failed to use database" << DbConfig::databaseName();
+        akDebug() << "Query error:" << query.lastError().text();
+        akDebug() << "Database error:" << db.lastError().text();
+        akDebug() << "Trying to create database now...";
+        if ( !query.exec( QLatin1String( "CREATE DATABASE akonadi" ) ) ) {
+          akError() << "Failed to create database";
+          akError() << "Query error:" << query.lastError().text();
+          akFatal() << "Database error:" << db.lastError().text();
+        }
+      }
+    } // make sure query is destroyed before we close the db
+    db.close();
+  }
+  QSqlDatabase::removeDatabase( initCon );
+}
+
 void AkonadiServer::stopDatabaseProcess()
 {
   if ( !mDatabaseProcess )
