@@ -95,7 +95,7 @@ void AkonadiConnection::run()
     connect( m_socket, SIGNAL( disconnected() ),
              this, SLOT( slotDisconnected() ), Qt::DirectConnection );
 
-    writeOut( "* OK Akonadi Almost IMAP Server [PROTOCOL 15]");
+    writeOut( "* OK Akonadi Almost IMAP Server [PROTOCOL 23]");
 
     m_streamParser = new ImapStreamParser( m_socket );
     exec();
@@ -112,7 +112,11 @@ void AkonadiConnection::slotDisconnected()
 
 void AkonadiConnection::slotNewData()
 {
-  while ( m_socket->bytesAvailable() > 0 || !m_streamParser->readRemainingData().isEmpty()) {
+  // On Windows, calling readLiteralPart() triggers the readyRead() signal recursively and leads to parse errors
+  if ( m_currentHandler )
+    return;
+    
+  while ( m_socket->bytesAvailable() > 0 || !m_streamParser->readRemainingData().isEmpty() ) {
     try {
       const QByteArray tag = m_streamParser->readString();
       // deal with stray newlines
@@ -134,21 +138,33 @@ void AkonadiConnection::slotNewData()
       }
     } catch ( const Akonadi::HandlerException &e ) {
       m_currentHandler->failureResponse( e.what() );
-      m_streamParser->readUntilCommandEnd(); //just eat the ending newline
+      try {
+        m_streamParser->readUntilCommandEnd(); //just eat the ending newline
+      } catch ( ... ) {}
     } catch ( const Akonadi::Exception &e ) {
-      m_currentHandler->failureResponse( QString::fromLatin1( e.type() )
+      if ( m_currentHandler ) {
+        m_currentHandler->failureResponse( QString::fromLatin1( e.type() )
           + QLatin1String( ": " ) + QString::fromLatin1( e.what()  ) );
-      m_streamParser->readUntilCommandEnd(); //just eat the ending newline
+      }
+      try {
+        m_streamParser->readUntilCommandEnd(); //just eat the ending newline
+      } catch ( ... ) {}
     } catch ( ... ) {
       akError() << "Unknown exception caught: " << akBacktrace();
-      m_currentHandler->failureResponse( "Unknown exception caught" );
-      m_streamParser->readUntilCommandEnd(); //just eat the ending newline
+      if ( m_currentHandler ) {
+        m_currentHandler->failureResponse( "Unknown exception caught" );
+      }
+      try {
+        m_streamParser->readUntilCommandEnd(); //just eat the ending newline
+      } catch ( ... ) {}
     }
     delete m_currentHandler;
     m_currentHandler = 0;
 
     if (m_streamParser->readRemainingData().startsWith('\n') || m_streamParser->readRemainingData().startsWith("\r\n"))
-      m_streamParser->readUntilCommandEnd(); //just eat the ending newline
+      try {
+        m_streamParser->readUntilCommandEnd(); //just eat the ending newline
+      } catch ( ... ) {}
   }
 }
 
