@@ -20,7 +20,6 @@
 
 #include "datastore.h"
 
-#include "agentmanagerinterface.h"
 #include "dbconfig.h"
 #include "dbinitializer.h"
 #include "dbupdater.h"
@@ -49,6 +48,7 @@
 #include <QtSql/QSqlError>
 #include <QtSql/QSqlField>
 #include <QtSql/QSqlQuery>
+#include "collectionqueryhelper.h"
 
 using namespace Akonadi;
 
@@ -142,19 +142,6 @@ DataStore * Akonadi::DataStore::self()
 }
 
 
-/* --- Flag ---------------------------------------------------------- */
-bool DataStore::appendFlag( const QString & name )
-{
-  if ( Flag::exists( name ) ) {
-    qDebug() << "Cannot insert flag " << name
-             << " because it already exists.";
-    return false;
-  }
-
-  Flag flag( name );
-  return flag.insert();
-}
-
 /* --- ItemFlags ----------------------------------------------------- */
 
 bool DataStore::setItemFlags( const PimItem &item, const QList<Flag> &flags )
@@ -192,22 +179,6 @@ bool DataStore::appendItemFlags( const PimItem &item, const QList<Flag> &flags,
   return true;
 }
 
-bool DataStore::appendItemFlags( const PimItem &item, const QList<QByteArray> &flags,
-                                 bool checkIfExists, const Collection &col )
-{
-  Flag::List list;
-  foreach ( const QByteArray &f, flags ) {
-    Flag flag = Flag::retrieveByName( QString::fromUtf8( f ) );
-    if ( !flag.isValid() ) {
-      flag = Flag( QString::fromUtf8( f ) );
-      if ( !flag.insert() )
-        return false;
-    }
-    list << flag;
-  }
-  return appendItemFlags( item, list, checkIfExists, col );
-}
-
 bool DataStore::removeItemFlags( const PimItem &item, const QList<Flag> &flags )
 {
   for ( int i = 0; i < flags.count(); ++i ) {
@@ -273,7 +244,7 @@ static bool recursiveSetResourceId( const Collection & collection, qint64 resour
   QueryBuilder qb( QueryBuilder::Update );
   qb.addTable( Collection::tableName() );
   qb.addValueCondition( Collection::parentIdColumn(), Query::Equals, collection.id() );
-  qb.updateColumnValue( Collection::resourceIdColumn(), resourceId );
+  qb.setColumnValue( Collection::resourceIdColumn(), resourceId );
   if ( !qb.exec() )
     return false;
   foreach ( const Collection &col, collection.children() ) {
@@ -283,38 +254,22 @@ static bool recursiveSetResourceId( const Collection & collection, qint64 resour
   return true;
 }
 
-bool Akonadi::DataStore::renameCollection( Collection & collection, qint64 newParent, const QByteArray & newName)
+bool Akonadi::DataStore::moveCollection( Collection & collection, const Collection &newParent )
 {
-  if ( collection.name() == newName && collection.parentId() == newParent )
+  if ( collection.parentId() == newParent.id() )
     return true;
 
-  if ( !m_dbOpened )
+  if ( !m_dbOpened || !newParent.isValid() )
     return false;
 
   int resourceId = collection.resourceId();
-  if ( newParent > 0 && collection.parentId() != newParent ) {
-    Collection parent = Collection::retrieveById( newParent );
-    resourceId = parent.resourceId();
-    if ( !parent.isValid() )
-      return false;
-
-    forever {
-      if ( parent.id() == collection.id() )
-        return false; // target is child of source
-      if ( parent.parentId() == 0 )
-        break;
-      parent = parent.parent();
-    }
-  }
-
-  SelectQueryBuilder<Collection> qb;
-  qb.addValueCondition( Collection::parentIdColumn(), Query::Equals, newParent );
-  qb.addValueCondition( Collection::nameColumn(), Query::Equals, newName );
-  if ( !qb.exec() || qb.result().count() > 0 )
+  const Collection source = collection.parent();
+  if ( newParent.id() > 0 ) // not root
+    resourceId = newParent.resourceId();
+  if ( !CollectionQueryHelper::canBeMovedTo ( collection, newParent ) )
     return false;
 
-  collection.setName( newName );
-  collection.setParentId( newParent );
+  collection.setParentId( newParent.id() );
   if ( collection.resourceId() != resourceId ) {
     collection.setResourceId( resourceId );
     if ( !recursiveSetResourceId( collection, resourceId ) )
@@ -324,8 +279,7 @@ bool Akonadi::DataStore::renameCollection( Collection & collection, qint64 newPa
   if ( !collection.update() )
     return false;
 
-  QList<QByteArray> changes = QList<QByteArray>() << "PARENT" << AKONADI_PARAM_NAME;
-  mNotificationCollector->collectionChanged( collection, changes );
+  mNotificationCollector->collectionMoved( collection, source );
   return true;
 }
 
@@ -433,16 +387,42 @@ bool DataStore::appendPimItem( QList<Part> & parts,
       (*it).setPimItemId( pimItem.id() );
       (*it).setDatasize( (*it).data().size() );
 
-      qDebug() << "Insert from DataStore::appendPimItem";
+//       qDebug() << "Insert from DataStore::appendPimItem";
       if( !PartHelper::insert(&(*it)) )
         return false;
     }
   }
 
+//   qDebug() << "appendPimItem: " << pimItem;
+
   mNotificationCollector->itemAdded( pimItem, collection, mimetype.name() );
   return true;
 }
 
+bool DataStore::unhidePimItem( PimItem &pimItem )
+{
+  if ( !m_dbOpened )
+    return false;
+
+  qDebug() << "DataStore::unhidePimItem(" << pimItem << ")";
+
+  // FIXME: This is inefficient. Using a bit on the PimItemTable record would probably be some orders of magnitude faster...
+  QList< QByteArray > parts;
+  parts << "ATR:HIDDEN";
+
+  return removeItemParts( pimItem, parts );
+}
+
+bool DataStore::unhideAllPimItems()
+{
+  if ( !m_dbOpened )
+    return false;
+
+  qDebug() << "DataStore::unhideAllPimItems()";
+
+  return PartHelper::remove( Part::nameFullColumnName(), QLatin1String( "ATR:HIDDEN" ) );
+}
+
 bool DataStore::cleanupPimItem( const PimItem &item )
 {
   if ( !item.isValid() )
@@ -563,6 +543,10 @@ bool Akonadi::DataStore::removeCollectionAttribute(const Collection & col, const
 
 void DataStore::debugLastDbError( const char* actionDescription ) const
 {
+  akError() << "Database error:" << actionDescription;
+  akError() << "  Last driver error:" << m_database.lastError().driverText();
+  akError() << "  Last database error:" << m_database.lastError().databaseText();
+
   Tracer::self()->error( "DataStore (Database Error)",
                          QString::fromLatin1( "%1\nDriver said: %2\nDatabase said:%3" )
                             .arg( QString::fromLatin1( actionDescription ) )
@@ -573,6 +557,11 @@ void DataStore::debugLastDbError( const char* actionDescription ) const
 
 void DataStore::debugLastQueryError( const QSqlQuery &query, const char* actionDescription ) const
 {
+  akError() << "Query error:" << actionDescription;
+  akError() << "  Last error message:" << query.lastError().text();
+  akError() << "  Last driver error:" << m_database.lastError().driverText();
+  akError() << "  Last database error:" << m_database.lastError().databaseText();
+
   Tracer::self()->error( "DataStore (Database Query Error)",
                          QString::fromLatin1( "%1: %2" )
                             .arg( QString::fromLatin1( actionDescription ) )
@@ -589,18 +578,6 @@ qint64 DataStore::uidNext() const
 }
 
 
-//static
-qint64 DataStore::lastInsertId( const QSqlQuery & query )
-{
-    QVariant v = query.lastInsertId();
-    if ( !v.isValid() ) return -1;
-    bool ok;
-    qint64 insertId = v.toLongLong( &ok );
-    if ( !ok ) return -1;
-    return insertId;
-}
-
-
 // static
 QString DataStore::dateTimeFromQDateTime( const QDateTime & dateTime )
 {
